import logging

from fastapi import APIRouter
from notebuild.tool.fastapi import add_api_routes, api_route
from notecoin.database.base import create_all, session
from notecoin.database.connect.base import BaseConnect
from notecoin.okex.database.client import OkexClientAccountBalance
from notecoin.okex.database.websocket import OkexWebsocketChannels
from notecoin.okex.websocket.channel import PublicChannel
from notecoin.okex.websocket.connect import PublicConnect


class WebsocketService(APIRouter):
    def __init__(self, connect: BaseConnect = None, prefix='websocket', *args, **kwargs):
        self.connect = connect or PublicConnect(channels=[PublicChannel.public_tickers().to_json()])
        super(WebsocketService, self).__init__(prefix=prefix, *args, **kwargs)
        add_api_routes(self)
        self.connect.run()

    def update_channels_db(self):
        details = session.query(OkexClientAccountBalance).filter(OkexClientAccountBalance.eqUsd > 10).all()
        create_all()
        for _detail in details:
            detail = _detail.json()
            if detail['ccy'] == 'USDT':
                continue
            session.merge(OkexWebsocketChannels(**detail))
        session.commit()
        create_all()
        return {"success": len(details)}

    @api_route('/update/channel', description="get value")
    def update_channels(self):
        self.update_channels_db()

        new_channels = session.query(OkexWebsocketChannels).all()
        new_channels = [eval(channel.channel_str) for channel in new_channels]

        old_channels_str = [str(channel) for channel in self.connect.channels]
        new_channels_str = [str(channel) for channel in new_channels]
        if new_channels_str is not None and len(new_channels_str) > 0:
            if len(list(set(old_channels_str) - set(new_channels_str))) == len(
                    list(set(new_channels_str) - set(old_channels_str))) == 0:
                logging.info(f"old:{old_channels_str},new:{new_channels_str}")
                self.connect.channels = new_channels
                self.connect.subscribe_restart()


WebsocketService()  # .update_channels_db()



# from notecoin.database.base import Base
# from sqlalchemy import BIGINT, Column, Float, String


# class BaseTable(object):
#     def json(self):
#         res = {k: v for k, v in self.__dict__.items() if not k.startswith("_")}
#         return res


# class OkexSocketPublicTickers(Base, BaseTable):
#     __tablename__ = 'okex_socket_public_tickets'
#     instType = Column(String(120), comment='产品类型', primary_key=True)
#     instId = Column(String(120), comment='产品ID', primary_key=True)

#     last = Column(Float(), comment='最新成交价')
#     lastSz = Column(Float(), comment='最新成交的数量')

#     askPx = Column(Float(), comment='卖一价')
#     askSz = Column(Float(), comment='卖一价对应的量')
#     bidPx = Column(Float(), comment='买一价')
#     bidSz = Column(Float(), comment='买一价对应的量')

#     open24h = Column(Float(), comment='24小时开盘价')
#     high24h = Column(Float(), comment='24小时最高价')
#     low24h = Column(Float(), comment='24小时最低价')
#     volCcy24h = Column(Float(), comment='24小时成交量，以计价货币为单位')
#     vol24h = Column(Float(), comment='24小时成交量，以交易货币为单位')

#     sodUtc0 = Column(Float(), comment='UTC 0时开盘价')
#     sodUtc8 = Column(Float(), comment='UTC+8时开盘价')

#     ts = Column(BIGINT(), comment='数据产生时间，Unix时间戳的毫秒数格式')

#     def __init__(self, *args, **kwargs):
#         self.instType = kwargs.get("instType", "SPOT")
#         self.instId = kwargs.get("instId", "")
#         self.last = kwargs.get("last", 0)
#         self.lastSz = kwargs.get("lastSz", 0)
#         self.askPx = kwargs.get("askPx", 0)
#         self.askSz = kwargs.get("askSz", 0)
#         self.bidPx = kwargs.get("bidPx", 0)

#         self.bidSz = kwargs.get("bidSz", 0)
#         self.open24h = kwargs.get("open24h", 0)
#         self.high24h = kwargs.get("high24h", 0)
#         self.low24h = kwargs.get("low24h", 0)
#         self.volCcy24h = kwargs.get("volCcy24h", 0)

#         self.vol24h = kwargs.get("vol24h", 0)
#         self.sodUtc0 = kwargs.get("sodUtc0", 0)
#         self.sodUtc8 = kwargs.get("sodUtc8", 0)
#         self.ts = kwargs.get("ts", 0)

#     def __repr__(self):
#         return f"instType{self.instType},instId:{self.instId}"
